@@ -120,13 +120,14 @@ typedef enum {
 } GSupplicantWpsState;
 
 typedef enum {
+	G_SUPPLICANT_PEER_SERVICES_CHANGED,
 	G_SUPPLICANT_PEER_GROUP_CHANGED,
 	G_SUPPLICANT_PEER_GROUP_STARTED,
 	G_SUPPLICANT_PEER_GROUP_FINISHED,
 	G_SUPPLICANT_PEER_GROUP_JOINED,
 	G_SUPPLICANT_PEER_GROUP_DISCONNECTED,
 	G_SUPPLICANT_PEER_GROUP_FAILED,
-} GSupplicantPeerGroupState;
+} GSupplicantPeerState;
 
 struct _GSupplicantSSID {
 	const void *ssid;
@@ -170,12 +171,26 @@ struct _GSupplicantScanParams {
 typedef struct _GSupplicantScanParams GSupplicantScanParams;
 
 struct _GSupplicantPeerParams {
+	bool master;
 	char *wps_pin;
 	char *path;
 };
 
 typedef struct _GSupplicantPeerParams GSupplicantPeerParams;
 
+struct _GSupplicantP2PServiceParams {
+	int version;
+	char *service;
+	unsigned char *query;
+	int query_length;
+	unsigned char *response;
+	int response_length;
+	unsigned char *wfd_ies;
+	int wfd_ies_length;
+};
+
+typedef struct _GSupplicantP2PServiceParams GSupplicantP2PServiceParams;
+
 /* global API */
 typedef void (*GSupplicantCountryCallback) (int result,
 						const char *alpha2,
@@ -229,6 +244,21 @@ int g_supplicant_interface_p2p_connect(GSupplicantInterface *interface,
 int g_supplicant_interface_p2p_disconnect(GSupplicantInterface *interface,
 					GSupplicantPeerParams *peer_params);
 
+int g_supplicant_interface_p2p_listen(GSupplicantInterface *interface,
+						int period, int interval);
+
+int g_supplicant_interface_p2p_add_service(GSupplicantInterface *interface,
+				GSupplicantInterfaceCallback callback,
+				GSupplicantP2PServiceParams *p2p_service_params,
+				void *user_data);
+
+int g_supplicant_interface_p2p_del_service(GSupplicantInterface *interface,
+				GSupplicantP2PServiceParams *p2p_service_params);
+
+int g_supplicant_set_widi_ies(GSupplicantP2PServiceParams *p2p_service_params,
+					GSupplicantInterfaceCallback callback,
+					void *user_data);
+
 int g_supplicant_interface_connect(GSupplicantInterface *interface,
 					GSupplicantSSID *ssid,
 					GSupplicantInterfaceCallback callback,
@@ -264,9 +294,11 @@ int g_supplicant_interface_set_country(GSupplicantInterface *interface,
 							void *user_data);
 bool g_supplicant_interface_has_p2p(GSupplicantInterface *interface);
 int g_supplicant_interface_set_p2p_device_config(GSupplicantInterface *interface,
-						const char *device_name);
+						const char *device_name,
+						const char *primary_dev_type);
 GSupplicantPeer *g_supplicant_interface_peer_lookup(GSupplicantInterface *interface,
 						const char *identifier);
+bool g_supplicant_interface_is_p2p_finding(GSupplicantInterface *interface);
 
 /* Network and Peer API */
 struct _GSupplicantNetwork;
@@ -295,12 +327,14 @@ const char *g_supplicant_peer_get_path(GSupplicantPeer *peer);
 const char *g_supplicant_peer_get_identifier(GSupplicantPeer *peer);
 const void *g_supplicant_peer_get_device_address(GSupplicantPeer *peer);
 const char *g_supplicant_peer_get_name(GSupplicantPeer *peer);
+const unsigned char *g_supplicant_peer_get_widi_ies(GSupplicantPeer *peer,
+								int *length);
 bool g_supplicant_peer_is_wps_pbc(GSupplicantPeer *peer);
 bool g_supplicant_peer_is_wps_pin(GSupplicantPeer *peer);
 bool g_supplicant_peer_is_in_a_group(GSupplicantPeer *peer);
 GSupplicantInterface *g_supplicant_peer_get_group_interface(GSupplicantPeer *peer);
 bool g_supplicant_peer_is_client(GSupplicantPeer *peer);
-bool g_supplicant_interface_is_p2p_finding(GSupplicantInterface *interface);
+bool g_supplicant_peer_has_requested_connection(GSupplicantPeer *peer);
 
 struct _GSupplicantCallbacks {
 	void (*system_ready) (void);
@@ -318,7 +352,8 @@ struct _GSupplicantCallbacks {
 	void (*peer_found) (GSupplicantPeer *peer);
 	void (*peer_lost) (GSupplicantPeer *peer);
 	void (*peer_changed) (GSupplicantPeer *peer,
-					GSupplicantPeerGroupState state);
+					GSupplicantPeerState state);
+	void (*peer_request) (GSupplicantPeer *peer);
 	void (*debug) (const char *str);
 };
 
