@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2012  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2013  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -22,8 +22,9 @@
 #ifndef __CONNMAN_PROVIDER_H
 #define __CONNMAN_PROVIDER_H
 
+#include <stdbool.h>
+
 #include <glib.h>
-#include <connman/types.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -103,7 +104,7 @@ int connman_provider_set_pac(struct connman_provider *provider,
 				const char *pac);
 int connman_provider_create_service(struct connman_provider *provider);
 int connman_provider_set_immutable(struct connman_provider *provider,
-						connman_bool_t immutable);
+						bool immutable);
 struct connman_provider *connman_provider_get(const char *identifier);
 void connman_provider_put(struct connman_provider *provider);
 int connman_provider_set_domain(struct connman_provider *provider,
@@ -133,7 +134,7 @@ struct connman_provider_driver {
 	int (*create) (DBusMessage *msg, connection_ready_cb callback);
 	int (*set_routes) (struct connman_provider *provider,
 				enum connman_provider_route_type type);
-	connman_bool_t (*check_routes) (struct connman_provider *provider);
+	bool (*check_routes) (struct connman_provider *provider);
 };
 
 int connman_provider_driver_register(struct connman_provider_driver *driver);
